import { IElementClasses, List } from "./models/list.model";
import { PageNode } from "./models/page-node.model";
import { PageElementNode } from "./nodes/page-element-node";

export class ElementClassList implements IElementClasses {
  get value(): string {
    return this.hostElement.classList.value ?? '';
  }

  private get hostElement(): Element {
    return this.pageElement.hostElement as Element;
  }

  constructor(
    private pageElement: PageElementNode,
  ) {}

  getClassNames(): string[] {
    return this.value.split(' ')
      .filter(className => {
        const includeSystemClasses = className.includes(this.pageElement.key);
        if (includeSystemClasses) {
          return false;
        }

        const isIgnored = this.pageElement.page.getOptions().ignoreCssClasses?.includes(className);
        return !isIgnored;
      });
  }

  init(): void {
    this.hostElement.classList.add(...(this.pageElement.schema.cssClasses || []));
  }

  add(...classNames: string[]): void {
    this.hostElement.classList.add(...classNames);
    this.updateSchemaClassList();
    this.pageElement.emit('view:change');
  }

  remove(...classNames: string[]): void {
    this.hostElement.classList.remove(...classNames);
    this.updateSchemaClassList();
    this.pageElement.emit('view:change');
  }

  toggle(className: string, force?: boolean): void {
    this.hostElement.classList.toggle(className, force);
    this.updateSchemaClassList();
    this.pageElement.emit('view:change');
  }

  private updateSchemaClassList(): void {
    this.pageElement.schema.cssClasses = this.getClassNames();
  }
}