import { StyleType } from "./models/style.model";
import { PageNode } from "./models/page-node.model";
import { PageElementNode } from "./nodes/page-element-node";
import { styleFactory } from "./styles/style-factory";
import { StyleBase } from "./styles/style-base";

export class ElementStyles {

  private stylesMap = new Map<string, StyleBase>();

  constructor(
    private pageElement: PageElementNode,
  ) {
  }

  init(): void {
    Object.values(StyleType).forEach(type => {
      const style = styleFactory(this.pageElement as PageElementNode, type)!;
      const schemaStyles = this.pageElement.schema.styles?.[type];
      this.stylesMap.set(type, style);
      if (schemaStyles) {
        style.setStyleRule(schemaStyles);
      }
    });
  }

  get(type: StyleType): StyleBase {
    return this.stylesMap.get(type)!;
  }
}