import { EventListener, IEventEmitter } from "./models/event-emitter.model";


export class EventEmitter implements IEventEmitter {
  private listenersMap = new Map<string, EventListener[]>();

  on<T>(eventNames: string, handler: EventListener<T>['handler'], options?: { autoUnlistenWhen: EventListener<T>['autoUnlistenWhen'] }): () => void {
    const eventNameList = eventNames.split(' ');
    eventNameList.forEach(eventName => {
      const listeners = this.getEventListeners(eventName);
      listeners.push({ handler, autoUnlistenWhen: options?.autoUnlistenWhen });
      this.listenersMap.set(eventName, listeners);
    });

    return () => this.unlisten(eventNames, handler);
  }

  once<T>(eventName: string, handler: EventListener<T>['handler']): () => void {
    const listeners = this.getEventListeners(eventName);
    listeners.push({ handler, once: true });

    return () => this.unlisten(eventName, handler);
  }

  emit<T>(eventName: string, payload?: T): void {
    const listeners = this.getEventListeners(eventName);
    listeners.forEach(listener => listener.handler(payload));
    this.listenersMap.set(
      eventName,
      listeners.filter(listener => !listener.once || (listener.autoUnlistenWhen && listener.autoUnlistenWhen()))
    );
  }

  private getEventListeners<T>(eventName: string): EventListener<T>[] {
    return this.listenersMap.get(eventName) ?? [];
  }

  private unlisten<T>(eventNames: string, handler: EventListener<T>['handler']): void {
    const eventNameList = eventNames.split(' ');
    eventNameList.forEach(eventName => {
      this.listenersMap.set(eventName, this.getEventListeners(eventName).filter(l => l.handler !== handler));
    });
  }
}