import { IPageBaseNode, IPageCustomNode, IPageElementNode, NodeType } from "../models/page-node.model";
import { PageBaseNode } from "./page-base-node";
import { IPage } from "../models/page.model";
import { Page } from "../page";
import { PageElementNode } from "./page-element-node";
import { SystemAttributes } from "../constants";
import { ElementAttributes } from "../element-attributes";
import { PageCustomNodeSchema, PageNodeSchema } from '../models/element-schema.model';
// import { cloneDeep } from "lodash";

export class PageCustomNode extends PageBaseNode<PageCustomNode> implements IPageCustomNode {
  override nativeElement!: HTMLElement;
  override nodeType: NodeType = NodeType.custom;
  override schema!: PageCustomNodeSchema;
  page!: Page;
  override hostElement!: HTMLElement;
  parentElement!: PageElementNode;

  constructor(
    schema: PageNodeSchema,
    page: IPage,
    parentElement: IPageElementNode | undefined,
  ) {
    super();
    this.page = page as Page;
    this.parentElement = parentElement as PageElementNode;
    this.schema = schema as PageCustomNodeSchema;
    this.attributes = new ElementAttributes(this);
    this.createCustomNode();
  }

  private createCustomNode(): void {
    const nodeTag = this.schema.tag.split('_')[1];
    if (!nodeTag) {
      console.warn('Page custom node tag was not specified');
      return;
    }
    super.createNode();
    
    const elRef = this.page.customElementFactory(this);
    this.nativeElement = elRef;
    this.hostElement = elRef;

    this.parentElement?.nativeElement?.appendChild(this.nativeElement);

    this.attributes.add({name: SystemAttributes.nodeKey, value: this.key});
    this.nativeElement.setAttribute(SystemAttributes.nodeKey, this.key);

    this.nativeElement.tabIndex = 1;
    this.nativeElement.style.display = 'block';
    this.page.nodeCreated(this);
  }

  updateSchema(schema: PageCustomNodeSchema): void {
    this.schema = schema;
    this.page.schemaChanged();
  }
}