import { IPageElementNode, IPageTextNode, NodeType } from "../models/page-node.model";
import { PageBaseNode } from "./page-base-node";
import { IPage } from "../models/page.model";
import { Page } from "../page";
import { ElementAttributes } from "../element-attributes";
import { PageNodeSchema, PageTextNodeSchema } from "../models/element-schema.model";

export class PageTextNode extends PageBaseNode<PageTextNode> implements IPageTextNode {
  override nativeElement!: Text;
  override nodeType: NodeType = NodeType.text;
  override schema!: PageTextNodeSchema;
  wrapper?: HTMLElement;
  page: Page;

  override get hostElement(): HTMLElement | Text {
    return this.wrapper || this.nativeElement;
  }

  constructor(
    schema: PageNodeSchema,
    page: IPage,
    public parentElement?: IPageElementNode | undefined,
  ) {
    super();
    this.page = page as Page;
    this.schema = schema as PageTextNodeSchema;
    this.attributes = new ElementAttributes(this);
    this.createTextNode();
  }

  wrap(wrapperTag: string): HTMLElement {
    this.parentElement?.nativeElement.removeChild(this.nativeElement);
    this.wrapper = this.page.documentRef.createElement(wrapperTag);
    this.wrapper.setAttribute('data-key', this.key);
    this.wrapper.appendChild(this.nativeElement);
    this.parentElement?.nativeElement.appendChild(this.hostElement);
    this.wrapper.tabIndex = 1;
    return this.wrapper;
  }

  getValue(): string {
    return this.schema.attributes?.find(attr => attr.name === 'text')?.value ?? '';
  }

  private createTextNode(): void {
    super.createNode();

    this.nativeElement = (this.page as Page).documentRef.createTextNode(this.getValue());
    this.parentElement?.nativeElement.appendChild(this.nativeElement);
    this.listenSelfEvents();
    this.page.nodeCreated(this);
  }

  private listenSelfEvents(): void {
    this.on('view:change', () => this.onViewChange());
  }

  private onViewChange(): void {
    const value = this.getValue();
    if (value !== this.nativeElement.textContent) {
      this.nativeElement.textContent = value;
    }
  }
}