import { PageElementNode } from "../nodes/page-element-node";
import { StyleBase } from "./style-base";
import { StyleType } from "../models/style.model";

export class CustomStyle extends StyleBase {
  override styleType: StyleType = StyleType.custom;
  override styleSheet: CSSStyleSheet;
  override selector: string;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
  }

  get(property: string): string | undefined {
    return this.rule.getProp(property);
  }

  getAll(): Record<string, string> {
    return this.rule.getAllProps() as Record<string, string>;
  }

  append(property: string, value: string, apply = true): void {
    this.rule.setProp(property, value);
    this.tryToApplyStyleRule(apply);
  }

  delete(property: string, apply = true): void {
    this.rule.deleteProp(property);

    this.tryToApplyStyleRule(apply);
  }

  attach(key: string): void {
    this.rule.attachProp(key);
    super.applyStyles();
  }

  detach(key: string): void {
    this.rule.detachProp(key);
    super.applyStyles();
  }

  detached(key: string): boolean {
    return this.rule.detached(key);
  }
}