import { StyleType } from "../models/style.model";
import { PageElementNode } from "../nodes/page-element-node";
import { isCssPropValueValid } from "../utils";
import { StyleBase } from './style-base';

export class LayerBlurStyle extends StyleBase {
  override styleType: StyleType = StyleType.layerBlur;
  override styleSheet: CSSStyleSheet;
  override selector: string;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
  }

  set(blur: string, apply = true): void {
    this.rule.setProp('filter', `blur(${blur})`);

    this.tryToApplyStyleRule(apply);
  }

  get(): string {
    return this.getEffectValue();
  }

  delete(apply = true): void {
    this.rule.deleteProp('filter');

    this.tryToApplyStyleRule(apply);
  }

  isValidValue(value: string): boolean {
    return isCssPropValueValid('filter', `blur(${value})`);
  }

  private getEffectValue(): string {
    if (!this.rule.getProp('filter')) {
      return '';
    }

    const match = this.rule.getProp<string>('filter')!.match(/\(([^)]+)\)/);
    return match![1];
  }
}