import { StyleType } from "../models/style.model";
import { PageElementNode } from "../nodes/page-element-node";
import { StyleBase } from "./style-base";

export interface Size {
  width?: string;
  height?: string;
}

export class SizeStyle extends StyleBase {
  override styleType: StyleType = StyleType.size;
  override styleSheet: CSSStyleSheet;
  override selector: string;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
  }

  setHeight(height: string, apply = true): void {
    this.rule.setProp('height', height);

    this.tryToApplyStyleRule(apply);
  }

  getHeight(): string | undefined {
    return this.rule.getProp('height');
  }

  deleteHeight(apply = true): void {
    this.rule.deleteProp('height');
    this.tryToApplyStyleRule(apply);
  }

  setWidth(width: string, apply = true): void {
    this.rule.setProp('width', width);
    this.tryToApplyStyleRule(apply);
  }

  getWidth(): string | undefined {
    return this.rule.getProp('width');
  }

  deleteWidth(apply = true): void {
    this.rule.deleteProp('width');
    this.tryToApplyStyleRule(apply);
  }
}