import { StyleType } from "../models/style.model";
import { PageElementNode } from "../nodes/page-element-node";
import { StyleBase } from "./style-base";

export type TextAlignment = 'left' | 'center' | 'right';
export type TextDecoration = 'line-through' | 'overline' | 'underline';

export class TextStyle extends StyleBase {
  override styleType: StyleType = StyleType.text;
  override styleSheet: CSSStyleSheet;
  override selector: string;

  constructor(public element: PageElementNode) {
    super();
    this.selector = `${this.styleType}-${this.element.key}`;
    this.styleSheet = this.element.page.getStyleSheet();
    this.setDefaults();
  }


  getColor(): string | undefined {
    return this.rule.getProp('color');
  }

  setColor(color: string, apply = true): void {
    this.rule.setProp('color', color);
    this.tryToApplyStyleRule(apply);
  }

  getLineHeight(): string | undefined {
    return this.rule.getProp('line-height');
  }

  setLineHeight(lineHeight: string, apply = true): void {
    this.rule.setProp('line-height', lineHeight);
    this.tryToApplyStyleRule(apply);
  }

  getAlignment(): TextAlignment | undefined {
    return this.rule.getProp('text-align');
  }

  setAlignment(alignment: TextAlignment, apply = true): void {
    this.rule.setProp('text-align', alignment);
    this.tryToApplyStyleRule(apply);
  }

  getFontSize(): string | undefined {
    return this.rule.getProp('font-size');
  }

  setFontSize(fontSize: string, apply = true): void {
    this.rule.setProp('font-size', fontSize);
    this.tryToApplyStyleRule(apply);
  }

  getDecoration(): TextDecoration | undefined {
    return this.rule.getProp('text-decoration');
  }

  setDecoration(decoration: TextDecoration, apply = true): void {
    this.rule.setProp('text-decoration', decoration);
    this.tryToApplyStyleRule(apply);
  }

  private setDefaults(): void {
    this.rule.setProp('color', '#000000');
    this.rule.setProp('line-height', 'auto');
    this.rule.setProp('text-align', 'left');
  }
}