const path = require('path');

module.exports = {
  // Set the mode to 'production' for optimized builds or 'development' for non-minified builds
  mode: 'production',

  // Entry point of your library
  entry: './src/index.ts',

  // Output configuration
  output: {
    // Output directory
    path: path.resolve(__dirname, 'dist'),
    // Output file name
    filename: 'html-page-builder.js',
    // Exported library name
    library: {
      name: 'HtmlPageBuilder',
      type: 'umd',
    },
    // Ensures compatibility across different environments (e.g., Node.js, web browsers)
    globalObject: 'this',
    // Clean the output directory before emit
    clean: true,
  },

  // Resolve file extensions
  resolve: {
    extensions: ['.ts', '.js'],
  },

  // Module processing rules
  module: {
    rules: [
      // Process TypeScript files
      {
        test: /\.tsx?$/,
        use: 'ts-loader',
        exclude: /node_modules/,
      },
      // Process CSS files as raw strings
      {
        test: /\.css$/,
        use: 'raw-loader',
      },
    ],
  },

  // External dependencies (if any)
  externals: {
    // Example: Exclude 'react' from the output bundle
    // 'react': 'react',
  },
};
